#include <msp430.h>

int other_master = 0;		// flag is set if another master is controlling the bus
int we_are_master = 0;

volatile int master_data = 0x01;
volatile int received_data = 0;

void arbitration_lost(void);
void config_slave_mode(void);
void config_master_mode(void);


void main(void){
	WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT

	P3SEL |= 0x06;                            // Assign I2C pins to USCI_B0 ************************
	P2DIR = 0xff;
	P2OUT = 0x00;
	__bis_SR_register(GIE);
	config_master_mode();

		while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent
		UCB0CTL1 |= UCTXSTT;             		// start condition
//		__bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
		if (UCB0STAT & UCALIFG)
			P2OUT |= BIT0;
//		__bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
		while(1);

}



#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIABOTX_ISR(void){
	if(we_are_master){
		UCB0TXBUF = master_data;
		UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
		    IFG2 &= ~UCB0TXIFG;                     // Clear USCI_B0 TX int flag
		    __bic_SR_register_on_exit(CPUOFF);      // Exit LPM0
	}
	else{
		received_data = UCB0RXBUF;
		__bic_SR_register_on_exit(CPUOFF);        // Exit LPM0
	}
}

#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void){

	switch(UCB0STAT & (UCSTPIFG + UCSTTIFG + UCALIFG)){		// checks for start and stop conditions, as well as arbitration lost
			case UCALIFG :
				arbitration_lost();
				UCB0STAT &= ~UCALIFG;				// clear arbitration lost flag
				break;
	/*		case UCSTTIFG :
	/			start_condition_detected();
	/			break;
			case UCSTPIFG :
				stop_condition_detected();
				break;
				*/
}
}
void arbitration_lost(void){
	config_slave_mode();
	we_are_master = 0;

}

void config_slave_mode(void){
	UCB0CTL1 |= UCSWRST;
	UCB0CTL0 &= ~UCMST + ~UCMM;
	UCB0CTL0 |= UCMODE_3 + UCSYNC;
	UCB0CTL1 &= ~UCTR;
	UCB0CTL1 |= UCSSEL_3;
	UCB0I2COA = 0x70;
	UCB0I2CIE |= UCALIE + UCSTTIE;
	IE2 |= UCB0TXIE + UCB0RXIE;
	UCB0CTL1 &= ~UCSWRST;
}

void config_master_mode(void){
	we_are_master = 1;
	UCB0CTL1 |= UCSWRST;
	UCB0CTL0 |= UCMM + UCMST + UCMODE_3 + UCSYNC;
	UCB0CTL1 |= UCSSEL_3 + UCTR;
	UCB0BR0 = 11;                             // fSCL = SMCLK/12 = ~100kHz
	UCB0BR1 = 0;
	UCB0CTL1 &= ~UCSWRST;
	UCB0I2CSA = 0x50;
	UCB0I2CIE |= UCALIE + UCSTTIE;
	IE2 |= UCB0TXIE;


}
